// Fill out your copyright notice in the Description page of Project Settings.


#include "BountyHunterCharacter.h"
#include "Kismet/GameplayStatics.h"
#include "EnhancedInputComponent.h"

void ABountyHunterCharacter::SpawnMainBounty(UEVGameInstance* GI)
{
	CurrentMainBountyIndex = GI->MainSaveGameInstanceRef->LastMainBountyIndexInOpenWorld;

	if (GI->MainSaveGameInstanceRef->CurrentMainBountyIndexSave > CurrentMainBountyIndex) CompleteCurrentMainBounty(GI);
		
	if (MainBountyClasses.IsEmpty() || MainBountyClasses.Num() <= CurrentMainBountyIndex|| !IsValid(MainBountyClasses[CurrentMainBountyIndex]))
	{
		AllBountiesCompleted();
		if (CurrentMainBountyIndex >= 2 && !(UGameplayStatics::GetCurrentLevelName(GetWorld()) == "EnemyAITest"))
		{
			UGameplayStatics::OpenLevel(GetWorld(), FName("EnemyAITest"));
		}
		return;
	}

	CurrentMainBounty = GetWorld()->SpawnActor<AMainBountyClass>(MainBountyClasses[CurrentMainBountyIndex]);
	const FAttachmentTransformRules AttachmentTransformRules(EAttachmentRule::SnapToTarget, true);
	CurrentMainBounty->AttachToComponent(GetRootComponent(), AttachmentTransformRules);

	UGameplayStatics::GetCurrentLevelName(GetWorld()) == OpenWorldLevelName ? CurrentMainBounty->SpawnOpenWorldCheckpoint() : CurrentMainBounty->SpawnCheckpoints();
}

void ABountyHunterCharacter::CompleteCurrentMainBounty(UEVGameInstance* GI)
{
	// Collect rewards for bounty and move onto next
	UE_LOG(LogTemp, Warning, TEXT("Would be collecting reward for completing bounty %d"), CurrentMainBountyIndex);

	FTransform PostBountySpawnTransform = MainBountyClasses[CurrentMainBountyIndex]->GetDefaultObject<AMainBountyClass>()->GetPlayerSpawnTransform();
	SetActorTransform(PostBountySpawnTransform);
	Cast<APlayerController>(GetController())->SetControlRotation(PostBountySpawnTransform.GetRotation().Rotator());
	
	CurrentMainBountyIndex = GI->MainSaveGameInstanceRef->CurrentMainBountyIndexSave;
	GI->MainSaveGameInstanceRef->LastMainBountyIndexInOpenWorld = CurrentMainBountyIndex;
	UGameplayStatics::SaveGameToSlot(GI->MainSaveGameInstanceRef, "MainSave", 0);
}

void ABountyHunterCharacter::AllBountiesCompleted()
{
	UE_LOG(LogTemp, Warning, TEXT("Game Completed!!!! No more Bounties left... or you forgot to set a bounty at one of the indexes oof"));
}

void ABountyHunterCharacter::BeginPlay()
{
	UEVGameInstance* GI = Cast<UEVGameInstance>(GetGameInstance());
	if (IsValid(GI->MainSaveGameInstanceRef))
	{
		SpawnMainBounty(GI);
		FTransform SpawnLoc = GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave;
		if (SpawnLoc.GetLocation() != FVector(0, 0, 0) && UGameplayStatics::GetCurrentLevelName(GetWorld()) == OpenWorldLevelName)
		{
			SetActorTransform(GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave);
			Cast<APlayerController>(GetController())->SetControlRotation(GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave.GetRotation().Rotator());
		} 
	}
	CreatePauseMenuTabs();
	Super::BeginPlay();
}

void ABountyHunterCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
	if (UEnhancedInputComponent* EnhancedInputComponent = CastChecked<UEnhancedInputComponent>(PlayerInputComponent))
	{
		EnhancedInputComponent->BindAction(PauseMenuAction, ETriggerEvent::Started, this, &ABountyHunterCharacter::TogglePauseMenu);
	}
	
	UE_LOG(LogTemp, Warning, TEXT("Setup player input on bounty hunter character"));

	Super::SetupPlayerInputComponent(PlayerInputComponent);
}

void ABountyHunterCharacter::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);
	if (UGameplayStatics::GetCurrentLevelName(GetWorld()) != OpenWorldLevelName) return;
	UEVGameInstance* GI = Cast<UEVGameInstance>(GetGameInstance());
	if (!IsValid(GI->MainSaveGameInstanceRef)) return;
	TArray<AActor*> LandingZones;
	ALandingZone* ClosestLandingZone = nullptr;
	double DistToClosestLandingZone;
	UGameplayStatics::GetAllActorsOfClass(GetWorld(), ALandingZone::StaticClass(), LandingZones);
	for (AActor* LZ_Actor : LandingZones)
	{
		ALandingZone* LZ = Cast<ALandingZone>(LZ_Actor);
     	double DistToLZ = FVector::Dist(LZ->GetExitTransform().GetLocation(), GetActorLocation());
		if (!IsValid(ClosestLandingZone))
		{
			ClosestLandingZone = LZ;
			DistToClosestLandingZone = DistToLZ;
			continue;
		}
		ClosestLandingZone = DistToClosestLandingZone >= DistToLZ ? LZ : ClosestLandingZone;
		DistToClosestLandingZone = DistToClosestLandingZone >= DistToLZ ? DistToLZ : DistToClosestLandingZone;
	}
	if (!IsValid(ClosestLandingZone)) return;
	GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave = ClosestLandingZone->GetExitTransform();
	UGameplayStatics::SaveGameToSlot(GI->MainSaveGameInstanceRef, "MainSave", 0);
}

void ABountyHunterCharacter::CreatePauseMenuTabs()
{
	BountiesTab = CreateWidget<UBountiesTab>(GetWorld(), BountyTabWidgetClass);
	FavoursTab = CreateWidget<UTabWidget>(GetWorld(), FavoursTabWidgetClass);
	NavigationTab = CreateWidget<UTabWidget>(GetWorld(), NavigationTabWidgetClass);
	SettingsTab = CreateWidget<UTabWidget>(GetWorld(), SettingsTabWidgetClass);
	QuitTab = CreateWidget<UQuitTab>(GetWorld(), QuitTabWidgetClass);
}

void ABountyHunterCharacter::TogglePauseMenu()
{
	const bool bOpenNewPauseMenu = !IsValid(PauseMenu);
	FInputModeUIOnly UIOnly;
	FInputModeGameOnly GameOnly;
	APlayerController* PC = Cast<APlayerController>(GetController());

	InPauseMenu = bOpenNewPauseMenu;
	PC->SetIgnoreMoveInput(bOpenNewPauseMenu);
	PC->SetIgnoreLookInput(bOpenNewPauseMenu);
	PC->SetShowMouseCursor(bOpenNewPauseMenu);
	if (!bOpenNewPauseMenu)
	{
		PauseMenu->RemoveFromParent();
		PauseMenu = nullptr;
		return;
	}
	PauseMenu = CreateWidget<UPauseMenuClass>(GetWorld(), PauseMenuWidgetClass);
	PauseMenu->AddToViewport(50);
	PauseMenu->OpenNewTab(BountiesTab);
	PauseMenu->OpenedBountiesTab.AddDynamic(this, &ABountyHunterCharacter::OpenBountiesTab);
	PauseMenu->OpenedFavoursTab.AddDynamic(this, &ABountyHunterCharacter::OpenFavoursTab);
	PauseMenu->OpenedNavigationTab.AddDynamic(this, &ABountyHunterCharacter::OpenNavigationTab);
	PauseMenu->OpenedSettingsTab.AddDynamic(this, &ABountyHunterCharacter::OpenSettingsTab);
	PauseMenu->OpenedQuitTab.AddDynamic(this, &ABountyHunterCharacter::OpenQuitTab);
}